Method: HDLRuby::High::Std#schannel
- Defined in:
- lib/HDLRuby/std/sequencer_channel.rb
#schannel(typ, size, &access) ⇒ Object
Creates an abstract channel over an accessing method. NOTE: Works like an enumerator or a memory access, but can be passed as generic arguments and generates a different enumerator per sequencer (memory access is identical for now though).
- +typ+ is the data type of the elements.
- +size+ is the number of elements.
- +access+ is the block implementing the access method.
86 87 88 |
# File 'lib/HDLRuby/std/sequencer_channel.rb', line 86 def schannel(typ,size,&access) return SequencerChannel.new(typ,size,&access) end |