Class: HDLRuby::High::Scope

Inherits:
Low::Scope show all
Includes:
HScope_missing, Hinner, Hmux, SingletonExtend, WithFullname
Defined in:
lib/HDLRuby/hruby_high.rb,
lib/HDLRuby/hruby_rsim.rb,
lib/HDLRuby/hruby_rcsim.rb,
lib/HDLRuby/hruby_rsim_vcd.rb,
lib/HDLRuby/hruby_high_fullname.rb

Overview

Describes a scope for a system type

Constant Summary collapse

High =
HDLRuby::High

Constants included from Hmissing

Hmissing::NAMES

Constants included from Low::Low2Symbol

Low::Low2Symbol::Low2SymbolPrefix, Low::Low2Symbol::Low2SymbolTable, Low::Low2Symbol::Symbol2LowTable

Instance Attribute Summary collapse

Attributes included from Low::Hparent

#parent

Instance Method Summary collapse

Methods included from Hmux

#mux

Methods included from HScope_missing

#h_missing, #method_missing

Methods included from Hmissing

#method_missing

Methods included from SingletonExtend

#eigen_extend

Methods included from Hinner

included

Methods inherited from Low::Scope

#add_behavior, #add_code, #add_connection, #add_inner, #add_program, #add_scope, #add_systemI, #add_systemT, #add_type, #blocks2seq!, #boolean_in_assign2select!, #break_concat_assigns!, #break_types!, #c_code_allocate, #casts_without_expression!, #cleanup!, #delete_all_behaviors!, #delete_all_connections!, #delete_behavior!, #delete_connection!, #delete_inner!, #delete_related!, #delete_scope!, #delete_systemI!, #delete_systemT!, #delete_type!, #delete_unless!, #each_behavior, #each_behavior_deep, #each_block_deep, #each_code, #each_connection, #each_connection_deep, #each_deep, #each_inner, #each_node_deep, #each_program, #each_scope, #each_scope_deep, #each_signal, #each_signal_deep, #each_statement_deep, #each_systemI, #each_systemT, #each_type, #eql?, #explicit_types!, #extract_behaviors!, #extract_connections!, #extract_declares!, #extract_port_assign!, #get_all_inners, #get_by_name, #get_code, #get_inner, #get_signal, #get_systemI, #get_systemT, #get_type, #has_behavior?, #has_code?, #has_connection?, #has_inner?, #has_program?, #has_scope?, #has_signal?, #has_systemI?, #has_systemT?, #has_type?, #hash, #initial_concat_to_timed!, #instance_port?, #last_behavior, #make_portw, #map_behaviors!, #map_connections!, #map_inners!, #map_scopes!, #map_systemIs!, #map_systemTs!, #map_types!, #mixblocks2seq!, #par_in_seq2seq!, #parent_system, #port_assign?, #portw2ref, #portw_name2sym, #replace_names!, #replace_names_subs!, #reverse_each_behavior, #select2case!, #signal2subs!, #sym2portw_name, #to_c, #to_ch, #to_hdr, #to_high, #to_upper_space!, #to_vhdl, #top_scope, #with_port!

Methods included from Low::ForceName

#extend_name!, #force_name!

Methods included from Low::Low2Symbol

#to_sym

Methods included from Low::Hparent

#absolute_ref, #hierarchy, #no_parent!, #scope

Constructor Details

#initialize(name = :"", systemT = nil, &ruby_block) ⇒ Scope

Creates a new scope with possible +name+. If the scope is a top scope of a system, this systemT is given by +systemT+.

The proc +ruby_block+ is executed for building the scope. If no block is provided, the scope is the top of a system and is filled by the instantiation procedure of the system.



930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
# File 'lib/HDLRuby/hruby_high.rb', line 930

def initialize(name = :"", systemT = nil, &ruby_block)
    # Initialize the scope structure
    super(name)

    # Initialize the set of grouped system instances.
    @groupIs = {}

    # Creates the namespace.
    @namespace = Namespace.new(self)

    # Register the scope if it is not the top scope of a system
    # (in which case the system has already be registered with
    # the same name).
    unless name.empty? or systemT then
        # Named scope, set the hdl-like access to the scope.
        obj = self # For using the right self within the proc
        High.space_reg(name) { obj }
    end

    # Initialize the set of exported inner signals and instances
    @exports = {}
    # Initialize the set of included systems.
    # @includes = {}
    @includes = []

    # Builds the scope if a ruby block is provided.
    self.build(&ruby_block) if block_given?
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method in the class HDLRuby::High::HScope_missing

Instance Attribute Details

#nameObject (readonly)

The name of the scope if any.



914
915
916
# File 'lib/HDLRuby/hruby_high.rb', line 914

def name
  @name
end

#namespaceObject (readonly)

The namespace



917
918
919
# File 'lib/HDLRuby/hruby_high.rb', line 917

def namespace
  @namespace
end

#rcscopeObject (readonly)

Extends the Scope class for hybrid Ruby-C simulation.



149
150
151
# File 'lib/HDLRuby/hruby_rcsim.rb', line 149

def rcscope
  @rcscope
end

#return_valueObject (readonly)

The return value when building the scope.



920
921
922
# File 'lib/HDLRuby/hruby_high.rb', line 920

def return_value
  @return_value
end

Instance Method Details

#add_export(name) ⇒ Object

Adds a +name+ to export.

NOTE: if the name do not corresponds to any inner signal nor instance, raise an exception.

Raises:



1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
# File 'lib/HDLRuby/hruby_high.rb', line 1001

def add_export(name)
    # Check the name.
    name = name.to_sym
    # Look for construct to make public.
    # Maybe it is an inner signals.
    inner = self.get_inner(name)
    if inner then
        # Yes set it as export.
        @exports[name] = inner
        return
    end
    # No, maybe it is an instance.
    instance = self.get_systemI(name)
    if instance then
        # Yes, set it as export.
        @exports[name] = instance
        return
    end
    # No, error.
    raise AnyError, "Invalid name for export: #{name}"
end

#add_groupI(name, *instances) ⇒ Object

Adds a group of system +instances+ named +name+.



966
967
968
969
970
971
972
973
974
975
976
977
978
# File 'lib/HDLRuby/hruby_high.rb', line 966

def add_groupI(name, *instances)
    # Ensure name is a symbol and is not already used for another
    # group.
    name = name.to_sym
    if @groupIs.key?(name)
        raise AnyError,
              "Group of system instances named #{name} already exist."
    end
    # Add the group.
    @groupIs[name.to_sym] = instances
    # Sets the parent of the instances.
    instances.each { |instance| instance.parent = self }
end

#add_inout(signal) ⇒ Object

Adds inout +signal+ in the current system.



1117
1118
1119
# File 'lib/HDLRuby/hruby_high.rb', line 1117

def add_inout(signal)
    self.parent.add_inout(signal)
end

#add_input(signal) ⇒ Object

Adds input +signal+ in the current system.



1107
1108
1109
# File 'lib/HDLRuby/hruby_high.rb', line 1107

def add_input(signal)
    self.parent.add_input(signal)
end

#add_output(signal) ⇒ Object

Adds output +signal+ in the current system.



1112
1113
1114
# File 'lib/HDLRuby/hruby_high.rb', line 1112

def add_output(signal)
    self.parent.add_output(signal)
end

#build(&ruby_block) ⇒ Object

Build the scope by executing +ruby_block+.

NOTE: used when the scope is not the top of a system.



1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
1083
# File 'lib/HDLRuby/hruby_high.rb', line 1063

def build(&ruby_block)
    # Set the namespace for buidling the scope.
    High.space_push(@namespace)
    # Build the scope.
    @return_value = High.top_user.instance_eval(&ruby_block)
    # res = High.top_user.instance_eval(&ruby_block)
    High.space_pop
    # # Now gain access to the result within the sub scope.
    # # if (res.is_a?(HRef)) then
    # if (res.is_a?(HExpression)) then
    #     High.space_push(@namespace)
    #     @return_value = res.type.inner(HDLRuby.uniq_name)
    #     @return_value <= res
    #     High.space_pop
    #     @return_value = RefObject.new(self,@return_value)
    # else
    #     @return_value = res
    # end
    # This will be the return value.
    @return_value
end

#build_top(base, *args) ⇒ Object

Builds the scope using +base+ as model scope with possible arguments +args+.

NOTE: Used by the instantiation procedure of a system.



1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
1100
1101
# File 'lib/HDLRuby/hruby_high.rb', line 1090

def build_top(base,*args)
    # Fills its namespace with the content of the base scope
    # (this latter may already contains access points if it has been
    #  opended for extension previously).
    @namespace.concat_namespace(base.namespace)
    High.space_push(@namespace)
    # Execute the instantiation block
    base.parent.each_instance_proc do |instance_proc|
        @return_value = High.top_user.instance_exec(*args,&instance_proc)
    end
    High.space_pop
end

#code(*content, &ruby_block) ⇒ Object

Declares a non-HDLRuby set of code chunks described by +content+ and completed from +ruby_block+ execution result. NOTE: content includes the events to activate the code on and a description of the code as a hash assotiating names to code text.



1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
# File 'lib/HDLRuby/hruby_high.rb', line 1189

def code(*content, &ruby_block)
    # Process the content.
    # Separate events from code chunks descriptions.
    events, chunks = content.partition {|elem| elem.is_a?(Event) }
    # Generates a large hash from the code.
    chunks = chunks.reduce(:merge)
    # Adds the result of the ruby block if any.
    if ruby_block then
        chunks.merge(HDLRuby::High.top_user.instance_eval(&ruby_block))
    end
    # Create the chunk objects.
    chunks = chunks.each.map do |name,content|
        content = [*content]
        # Process the lumps
        content.map! do |lump|
            lump.respond_to?(:to_expr) ? lump.to_expr : lump
        end
        Chunk.new(name,*content)
    end
    # Create the code object.
    res = Code.new
    # Adds the events.
    events.each(&res.method(:add_event))
    # Adds the chunks.
    chunks.each(&res.method(:add_chunk))
    # Adds the resulting code to the current scope.
    HDLRuby::High.top_user.add_code(res)
    # Return the resulting code
    return res
end

#cur_systemObject

Gets the current system.



1415
1416
1417
# File 'lib/HDLRuby/hruby_high.rb', line 1415

def cur_system
    return HDLRuby::High.cur_system
end

#each_export(&ruby_block) ⇒ Object

Iterates over the exported constructs.

Returns an enumerator if no ruby block is given.



1026
1027
1028
1029
1030
1031
1032
1033
# File 'lib/HDLRuby/hruby_high.rb', line 1026

def each_export(&ruby_block)
    # No ruby block? Return an enumerator.
    return to_enum(:each_export) unless ruby_block
    # A block? Apply it on each input signal instance.
    @exports.each_value(&ruby_block)
    # And apply on the sub scopes if any.
    @scopes.each {|scope| scope.each_export(&ruby_block) }
end

#each_groupI(&ruby_block) ⇒ Object

Iterates over the group of system instances.

Returns an enumerator if no ruby block is given.



990
991
992
993
994
995
# File 'lib/HDLRuby/hruby_high.rb', line 990

def each_groupI(&ruby_block)
    # No ruby block? Return an enumerator.
    return to_enum(:each_groupI) unless ruby_block
    # A block? Apply it on each input signal instance.
    @groupIs.each(&ruby_block)
end

#each_included(&ruby_block) ⇒ Object

Iterates over the included systems.



1036
1037
1038
1039
1040
1041
1042
1043
1044
# File 'lib/HDLRuby/hruby_high.rb', line 1036

def each_included(&ruby_block)
    # No ruby block? Return an enumerator.
    return to_enum(:each_included) unless ruby_block
    # A block? Apply it on each included system.
    # @includes.each_value(&ruby_block)
    @includes.each(&ruby_block)
    # And apply on the sub scopes if any.
    @scopes.each {|scope| scope.each_included(&ruby_block) }
end

#export(*names) ⇒ Object

Sets the constructs corresponding to +names+ as exports.



1354
1355
1356
# File 'lib/HDLRuby/hruby_high.rb', line 1354

def export(*names)
    names.each {|name| self.add_export(name) }
end

#fill_low(scopeL) ⇒ Object

Fills a low level scope with self's contents.

NOTE: name conflicts are treated in the current NameStack state.



1493
1494
1495
1496
1497
1498
1499
1500
1501
1502
1503
1504
1505
1506
1507
1508
1509
1510
1511
1512
1513
1514
1515
1516
1517
1518
1519
1520
1521
1522
1523
1524
1525
1526
1527
1528
1529
1530
1531
1532
1533
1534
1535
1536
1537
1538
1539
1540
1541
# File 'lib/HDLRuby/hruby_high.rb', line 1493

def fill_low(scopeL)
    # Adds the content of its included systems.
    # @includes.each_value {|system| system.scope.fill_low(scopeL) }
    @includes.each {|system| system.scope.fill_low(scopeL) }
    # Adds the declared local system types.
    # NOTE: in the current version of HDLRuby::High, there should not
    # be any of them (only eigen systems are real system types).
    self.each_systemT { |systemT| scopeL.add_systemT(systemT.to_low) }
    # Adds the local types.
    self.each_type { |type| scopeL.add_type(type.to_low) }
    # Adds the inner scopes.
    self.each_scope { |scope| scopeL.add_scope(scope.to_low) }
    # Adds the inner signals.
    self.each_inner { |inner| scopeL.add_inner(inner.to_low) }
    # Adds the instances.
    # Single ones.
    self.each_systemI do |systemI|
        # puts "Filling with systemI=#{systemI.name}"
        systemI_low = scopeL.add_systemI(systemI.to_low)
        # Also add the eigen system to the list of local systems.
        scopeL.add_systemT(systemI_low.systemT)
    end
    # Grouped ones.
    self.each_groupI do |name,systemIs|
        systemIs.each.with_index { |systemI,i|
            # Sets the name of the system instance
            # (required for conversion of further accesses).
            # puts "systemI.respond_to?=#{systemI.respond_to?(:name=)}"
            systemI.name = name.to_s + "[#{i}]"
            # And convert it to low
            systemI_low = scopeL.add_systemI(systemI.to_low())
            # Also add the eigen system to the list of local systems.
            scopeL.add_systemT(systemI_low.systemT)
        }
    end
    # Adds the programs.
    self.each_program { |prog| scopeL.add_program(prog.to_low) }
    # Adds the code chunks.
    self.each_code { |code| scopeL.add_code(code.to_low) }
    # Adds the connections.
    self.each_connection { |connection|
        # puts "connection=#{connection}"
        scopeL.add_connection(connection.to_low)
    }
    # Adds the behaviors.
    self.each_behavior { |behavior|
        scopeL.add_behavior(behavior.to_low)
    }
end

#fullnameObject

Returns the name of the signal with its hierarchy.



369
370
371
372
# File 'lib/HDLRuby/hruby_rsim.rb', line 369

def fullname
    @fullname ||= self.parent.fullname + ":" + self.name.to_s
    return @fullname
end

#get_groupI(name) ⇒ Object

Access a group of system instances by +name+.

NOTE: the result is a copy of the group for avoiding side effects.



983
984
985
# File 'lib/HDLRuby/hruby_high.rb', line 983

def get_groupI(name)
    return @groupIs[name.to_sym].clone
end

#get_vars_with_fullname(vars_with_fullname = {}) ⇒ Object

Gets the VCD variables with their long name.



204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
# File 'lib/HDLRuby/hruby_rsim_vcd.rb', line 204

def get_vars_with_fullname(vars_with_fullname = {})
    # # Adds the inner signals.
    # self.each_inner do |sig|
    #     vars_with_fullname[sig] = HDLRuby::High.vcd_name(sig.fullname)
    # end
    # Recurse on the inner signals.
    self.each_inner do |sig|
        sig.get_vars_with_fullname(vars_with_fullname)
        # vars_with_fullname[sig] = HDLRuby::High.vcd_name(sig.fullname)
    end
    # Recurse on the behaviors' blocks
    self.each_behavior do |beh|
        beh.block.get_vars_with_fullname(vars_with_fullname)
    end
    # Recurse on the systemI's Eigen system.
    self.each_systemI do |sys|
        sys.systemT.get_vars_with_fullname(vars_with_fullname)
    end
    # Recurse on the subscopes.
    self.each_scope do |scope|
        scope.get_vars_with_fullname(vars_with_fullname)
    end
    return vars_with_fullname
end

#get_vars_with_idstr(vars_with_idstr = {}) ⇒ Object

Gets the VCD variables with their id string.



230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
# File 'lib/HDLRuby/hruby_rsim_vcd.rb', line 230

def get_vars_with_idstr(vars_with_idstr = {})
    # # Adds the inner signals.
    # self.each_inner do |sig|
    #     vars_with_idstr[sig] = HDLRuby::High.vcd_idstr(sig)
    # end
    # Recurse on the inner signals.
    self.each_inner do |sig|
        sig.get_vars_with_idstr(vars_with_idstr)
    end
    # Recurse on the behaviors' blocks
    self.each_behavior do |beh|
        beh.block.get_vars_with_idstr(vars_with_idstr)
    end
    # Recurse on the systemI's Eigen system.
    self.each_systemI do |sys|
        sys.systemT.get_vars_with_idstr(vars_with_idstr)
    end
    # Recurse on the subscopes.
    self.each_scope do |scope|
        scope.get_vars_with_idstr(vars_with_idstr)
    end
    return vars_with_idstr
end

#hcase(value) ⇒ Object

Creates a new case statement with a +value+ used for deciding which block to execute.

NOTE:

  • the when part is defined through the hwhen method.
  • a new behavior is created to enclose the hcase.


1329
1330
1331
1332
1333
# File 'lib/HDLRuby/hruby_high.rb', line 1329

def hcase(value)
    self.par do
        hcase(value)
    end
end

#helse(mode = nil, &ruby_block) ⇒ Object

Sets the block executed when the condition is not met to the block in +mode+ generated by the execution of +ruby_block+.

Can only be used once.

NOTE: added to the hif of the last behavior.



1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
# File 'lib/HDLRuby/hruby_high.rb', line 1294

def helse(mode = nil, &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # There is a ruby_block: the helse is assumed to be with
    # the last statement of the last behavior.
    statement = self.last_behavior.last_statement
    # Completes the hif or the hcase statement.
    unless statement.is_a?(If) or statement.is_a?(Case) then
        raise AnyError, "Error: helse statement without hif nor hcase (#{statement.class})."
    end
    statement.helse(mode, &ruby_block)
end

#helsif(condition, mode = nil, &ruby_block) ⇒ Object

Sets the condition check when the condition is not met to the block, with a +condition+ that when met lead to the execution of the block in +mode+ generated by the +ruby_block+.



1310
1311
1312
1313
1314
1315
1316
1317
1318
1319
1320
1321
# File 'lib/HDLRuby/hruby_high.rb', line 1310

def helsif(condition, mode = nil, &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # There is a ruby_block: the helse is assumed to be with
    # the last statement of the last behavior.
    statement = self.last_behavior.last_statement
    # Completes the hif statement.
    unless statement.is_a?(If) then
        raise AnyError, "Error: helsif statement without hif (#{statement.class})."
    end
    statement.helsif(condition, mode, &ruby_block)
end

#hif(condition, mode = nil, &ruby_block) ⇒ Object

Creates a new if statement with a +condition+ that when met lead to the execution of the block in +mode+ generated by the +ruby_block+.

NOTE:

  • the else part is defined through the helse method.
  • a behavior is created to enclose the hif.


1280
1281
1282
1283
1284
1285
1286
# File 'lib/HDLRuby/hruby_high.rb', line 1280

def hif(condition, mode = nil, &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    self.par do
        hif(condition,mode,&ruby_block)
    end
end

#hwhen(match, mode = nil, &ruby_block) ⇒ Object

Sets the block of a case structure executed when the +match+ is met to the block in +mode+ generated by the execution of +ruby_block+.

Can only be used once.



1339
1340
1341
1342
1343
1344
1345
1346
1347
1348
1349
1350
# File 'lib/HDLRuby/hruby_high.rb', line 1339

def hwhen(match, mode = nil, &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # There is a ruby_block: the helse is assumed to be with
    # the last statement of the last behavior.
    statement = @behaviors.last.last_statement
    # Completes the hcase statement.
    unless statement.is_a?(Case) then
        raise AnyError, "Error: hwhen statement without hcase (#{statement.class})."
    end
    statement.hwhen(match, mode, &ruby_block)
end

#include(system, *args) ⇒ Object

Include a +system+ type with possible +args+ instanciation arguments.



1360
1361
1362
1363
1364
1365
1366
1367
1368
1369
1370
1371
1372
1373
1374
1375
1376
1377
1378
1379
1380
1381
1382
1383
1384
1385
1386
1387
1388
1389
1390
1391
1392
1393
1394
1395
1396
1397
1398
1399
1400
1401
1402
1403
# File 'lib/HDLRuby/hruby_high.rb', line 1360

def include(system,*args)
    # if @includes.key?(system.name) then
    #     raise AnyError, "Cannot include twice the same system: #{system}"
    # end
    if @includes.include?(system) then
        raise AnyError, "Cannot include twice the same system: #{system}"
    end
    # # puts "Include system=#{system.name}"
    # # Save the name of the included system, it will serve as key
    # # for looking for the included expanded version.
    # include_name = system.name
    # Expand the system to include
    system = system.expand(:"",*args)
    # Add the included system interface to the current one.
    if self.parent.is_a?(SystemT) then
        space = self.namespace
        # Interface signals
        # puts "i_name=#{i_name} @to_includes=#{@to_includes.size}"
        # system.each_signal_with_included do |signal|
        system.each_signal_all_with_included do |signal|
            # puts "signal=#{signal.name}"
            space.send(:define_singleton_method,signal.name) do
                signal
            end
        end
        # Exported objects
        system.each_export do |export|
            # puts "export=#{export.name}"
            space.send(:define_singleton_method,export.name) do
                export
            end
        end
        # Adds the task to execute on the instance.
        system.each_on_instance do |task|
            self.parent.on_instance(&task)
        end
    end
    # Adds it the list of includeds
    # @includes[include_name] = system
    @includes << system

    # puts "@includes=#{@includes}"
    
end

#init_sim(systemT) ⇒ Object

Initialize the simulation for system +systemT+.



336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
# File 'lib/HDLRuby/hruby_rsim.rb', line 336

def init_sim(systemT)
    # Recurse on the inner signals.
    self.each_inner { |sig| sig.init_sim(systemT) }
    # Recurse on the behaviors.
    self.each_behavior { |beh| beh.init_sim(systemT) }
    # Recurse on the systemI.
    self.each_systemI { |sys| sys.init_sim(systemT) }
    # Recurse on the connections.
    # self.each_connection { |cnx| cnx.init_sim(systemT) }
    self.each_connection do |cnx|
        # Connection to a real expression?
        if !cnx.right.is_a?(RefObject) then
            # Yes.
            cnx.init_sim(systemT)
        else
            # No, maybe the reverse connection is also required.
            # puts "cnx.left.object=#{cnx.left.object.fullname} cnx.right.object=#{cnx.right.object.fullname}"
            cnx.init_sim(systemT)
            if cnx.left.is_a?(RefObject) then
                sigL = cnx.left.object
                prtL = sigL.parent
                if prtL.is_a?(SystemT) and prtL.each_inout.any?{|e| e.object_id == sigL.object_id} then
                    # puts "write to right with sigL=#{sigL.fullname}."
                    Connection.new(cnx.right.clone,cnx.left.clone).init_sim(systemT)
                end
            end
        end
    end
    # Recurse on the sub scopes.
    self.each_scope { |sco| sco.init_sim(systemT) }
end

#inout(*names) ⇒ Object

Declares high-level bit inout signals named +names+ in the current system.



1169
1170
1171
# File 'lib/HDLRuby/hruby_high.rb', line 1169

def inout(*names)
    self.parent.inout(*names)
end

#input(*names) ⇒ Object

Declares high-level bit input signals named +names+ in the current system.



1157
1158
1159
# File 'lib/HDLRuby/hruby_high.rb', line 1157

def input(*names)
    self.parent.input(*names)
end

#make_inouts(type, *names) ⇒ Object

Creates and adds a set of inouts typed +type+ from a list of +names+ in the current system.

NOTE: a name can also be a signal, is which case it is duplicated.



1141
1142
1143
# File 'lib/HDLRuby/hruby_high.rb', line 1141

def make_inouts(type, *names)
    self.parent.make_inouts(type,*names)
end

#make_inputs(type, *names) ⇒ Object

Creates and adds a set of inputs typed +type+ from a list of +names+ in the current system.

NOTE: a name can also be a signal, is which case it is duplicated.



1125
1126
1127
# File 'lib/HDLRuby/hruby_high.rb', line 1125

def make_inputs(type, *names)
    self.parent.make_inputs(type,*names)
end

#make_outputs(type, *names) ⇒ Object

Creates and adds a set of outputs typed +type+ from a list of +names+ in the current system.

NOTE: a name can also be a signal, is which case it is duplicated.



1133
1134
1135
# File 'lib/HDLRuby/hruby_high.rb', line 1133

def make_outputs(type, *names)
    self.parent.make_outputs(type,*names)
end

#merge_included(systemT) ⇒ Object

Merge the included systems interface in +systemT+ NOTE: incompatible with further to_low transformation.



1425
1426
1427
1428
1429
1430
1431
1432
1433
1434
1435
1436
1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
1447
1448
1449
1450
1451
1452
1453
1454
1455
1456
1457
1458
1459
1460
1461
1462
1463
1464
1465
1466
1467
1468
1469
1470
1471
1472
1473
1474
1475
1476
1477
1478
# File 'lib/HDLRuby/hruby_high.rb', line 1425

def merge_included(systemT)
    # puts "merge_included for scope=#{self.name} with behaviors=#{self.each_behavior.count}" 
    # Recurse on the sub.
    self.each_scope {|scope| scope.merge_included(systemT) }
    # Include for current scope.
    self.each_included do |included|
        included.merge_included!
        # Adds its interface signals.
        included.each_input do |input|
            input.no_parent!
            systemT.add_input(input)
        end
        included.each_output do |output|  
            output.no_parent!
            systemT.add_output(output)
        end
        included.each_inout do |inout|  
            inout.no_parent!
            systemT.add_inout(inout)
        end
        # Adds its behaviors.
        included.scope.each_behavior do |beh|
            beh.no_parent!
            systemT.scope.add_behavior(beh)
        end
        # Adds its connections.
        included.scope.each_connection do |cx|
            cx.no_parent!
            systemT.scope.add_connection(cx)
        end
        # Adds its sytem instances.
        included.scope.each_systemI do |sys|
            sys.no_parent!
            systemT.scope.add_systemI(sys)
        end
        # Adds its code.
        included.scope.each_code do |code|
            code.no_parent!
            systemT.scope.add_code(code)
        end
        # Adds its subscopes.
        included.scope.each_scope do |scope|
            # Do not override scopes with same name since it is prioritary!
            next if !scope.name.empty? && systemT.scope.each_scope.find {|sc| sc.name == scope.name}
            scope.no_parent!
            systemT.scope.add_scope(scope)
        end
        # Add its inner signals.
        included.scope.each_inner do |inner|
            inner.no_parent!
            systemT.scope.add_inner(inner)
        end
    end
end

#merge_included!Object

Merge the included systems interface in system instances. NOTE: incompatible with further to_low transformation.



1482
1483
1484
1485
1486
1487
# File 'lib/HDLRuby/hruby_high.rb', line 1482

def merge_included!
    # Recurse on the sub.
    self.each_scope {|scope| scope.merge_included! }
    # Merge in the system instances.
    self.each_systemI {|systemI| systemI.systemT.merge_included! }
end

#open(&ruby_block) ⇒ Object

Opens for extension.

NOTE: actually executes +ruby_block+ in the context.



1050
1051
1052
1053
1054
1055
1056
1057
# File 'lib/HDLRuby/hruby_high.rb', line 1050

def open(&ruby_block)
    High.space_push(@namespace)
    res = High.top_user.instance_eval(&ruby_block)
    High.space_pop
    # Return the result of the execution so that it can be used
    # as an expression
    res
end

#output(*names) ⇒ Object

Declares high-level bit output signals named +names+ in the current system.



1163
1164
1165
# File 'lib/HDLRuby/hruby_high.rb', line 1163

def output(*names)
    self.parent.output(*names)
end

#par(*events, &ruby_block) ⇒ Object

Declares a high-level parallel behavior activated on a list of +events+, and built by executing +ruby_block+.



1252
1253
1254
1255
1256
1257
1258
1259
1260
1261
# File 'lib/HDLRuby/hruby_high.rb', line 1252

def par(*events, &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # Preprocess the events.
    events.map! do |event|
        event.respond_to?(:to_event) ? event.to_event : event
    end
    # Create and add the resulting behavior.
    self.add_behavior(Behavior.new(:par,*events,&ruby_block))
end

#program(lang, func, &ruby_block) ⇒ Object

Declares a program in language +lang+ with start function named +func+ and built through +ruby_block+.



1175
1176
1177
1178
1179
1180
1181
1182
# File 'lib/HDLRuby/hruby_high.rb', line 1175

def program(lang, func, &ruby_block)
    # Create the program.
    prog = Program.new(lang, func, &ruby_block)
    # Adds the resulting program to the current scope.
    HDLRuby::High.top_user.add_program(prog)
    # Return the resulting program
    return prog
end

#seq(*events, &ruby_block) ⇒ Object

Declares a high-level sequential behavior activated on a list of +events+, and built by executing +ruby_block+.



1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
# File 'lib/HDLRuby/hruby_high.rb', line 1239

def seq(*events, &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # Preprocess the events.
    events.map! do |event|
        event.respond_to?(:to_event) ? event.to_event : event
    end
    # Create and add the resulting behavior.
    self.add_behavior(Behavior.new(:seq,*events,&ruby_block))
end

#show_hierarchy(vcdout) ⇒ Object

Shows the hierarchy of the variables.



168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
# File 'lib/HDLRuby/hruby_rsim_vcd.rb', line 168

def show_hierarchy(vcdout)
    # puts "show_hierarchy for scope=#{self}"
    # Shows the current level of hierarchy if there is a name.
    ismodule = false
    if  !self.name.empty? && !self.parent.is_a?(SystemT) then
        vcdout << "$scope module #{HDLRuby::High.vcd_name(self.fullname)} $end\n"
        ismodule = true
    end
    # Shows the inner signals.
    self.each_inner do |sig|
        sig.show_hierarchy(vcdout)
        # # puts "showing inner signal #{HDLRuby::High.vcd_name(sig.fullname)}"
        # vcdout << "$var wire #{sig.type.width} "
        # # vcdout << "#{HDLRuby::High.vcd_name(sig.fullname)} "
        # vcdout << "#{HDLRuby::High.vcd_idstr(sig)} "
        # vcdout << "#{HDLRuby::High.vcd_name(sig.name)} $end\n"
    end
    # Recurse on the behaviors' blocks
    self.each_behavior do |beh|
        beh.block.show_hierarchy(vcdout)
    end
    # Recurse on the systemI's Eigen system.
    self.each_systemI do |sys|
        sys.systemT.show_hierarchy(vcdout)
    end
    # Recurse on the subscopes.
    self.each_scope do |scope|
        scope.show_hierarchy(vcdout)
    end
    # Close the current level of hierarchy if there is a name.
    if ismodule then
        vcdout << "$upscope $end\n"
    end
end

#sub(name = :"", &ruby_block) ⇒ Object

Declares a sub scope with possible +name+ and built from +ruby_block+.



1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
# File 'lib/HDLRuby/hruby_high.rb', line 1221

def sub(name = :"", &ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # Creates the new scope.
    # scope = Scope.new(name,&ruby_block)
    scope = Scope.new(name)
    # Add it
    self.add_scope(scope)
    # Build it.
    scope.build(&ruby_block)
    # puts "self=#{self}"
    # puts "self scopes=#{self.each_scope.to_a.join(",")}"
    # Use its return value
    return scope.return_value
end

#timed(&ruby_block) ⇒ Object

Declares a high-level timed behavior built by executing +ruby_block+. By default, timed behavior are sequential.



1265
1266
1267
1268
1269
1270
# File 'lib/HDLRuby/hruby_high.rb', line 1265

def timed(&ruby_block)
    # Ensure there is a block.
    ruby_block = proc {} unless block_given?
    # Create and add the resulting behavior.
    self.add_behavior(TimeBehavior.new(:seq,&ruby_block))
end

#to_low(low_name = nil) ⇒ Object

Converts the scope to HDLRuby::Low. +name+ is the name of the system containing the new low scope in case of top scope, should be used as name for it. NOTE: by convention, the name of the top scope is the name of the system.



1548
1549
1550
1551
1552
1553
1554
1555
1556
1557
1558
1559
1560
1561
1562
1563
1564
1565
1566
1567
1568
1569
1570
# File 'lib/HDLRuby/hruby_high.rb', line 1548

def to_low(low_name = nil)
    # Create the resulting low scope.
    # scopeL = HDLRuby::Low::Scope.new()
    low_name = self.name unless low_name
    scopeL = HDLRuby::Low::Scope.new(low_name)
    # # For debugging: set the source high object 
    # scopeL.properties[:low2high] = self.hdr_id
    # self.properties[:high2low] = scopeL

    # Push the private namespace for the low generation.
    High.space_push(@namespace)
    # Pushes on the name stack for converting the internals of
    # the system.
    High.names_push
    # Adds the content of the actual system.
    self.fill_low(scopeL)
    # Restores the name stack.
    High.names_pop
    # Restores the namespace stack.
    High.space_pop
    # Return theresulting system.
    return scopeL
end

#to_rcsim(rcowner) ⇒ Object

Generate the C description of the scope comming from object whose C description is +rcowner+



153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
# File 'lib/HDLRuby/hruby_rcsim.rb', line 153

def to_rcsim(rcowner)
    # puts "to_rcsim for scope=#{self}"
    # Create the scope C object.
    @rcscope = RCSim.rcsim_make_scope(self.name.to_s)

    # Set the owner.
    RCSim.rcsim_set_owner(@rcscope,rcowner)

    # Of the scope is a son of a SystemT, the owner of the sub objects
    # will be this systemT. Otherwise, it is the scope.
    subowner = self.parent.is_a?(SystemT) ? rcowner : @rcscope

    # Create and add the inner signals.
    if self.each_inner.any? then
        RCSim.rcsim_add_scope_inners(@rcscope,self.each_inner.map do|sig|
            # sig.to_rcsim(@rcscope)
            sig.to_rcsim(subowner)
        end)
    end
    
    # Create and add the system instances.
    if self.each_systemI.any? then
        RCSim.rcsim_add_scope_systemIs(@rcscope,
                                       self.each_systemI.map do |sys|
            # sys.to_rcsim(@rcscope)
            sys.to_rcsim(subowner)
        end)
    end

    # Create and add the sub scopes.
    if self.each_scope.any? then
        RCSim.rcsim_add_scope_scopes(@rcscope,self.each_scope.map do|sub|
            # sub.to_rcsim(@rcscope)
            sub.to_rcsim(subowner)
        end)
    end

    # Create and add the behaviors and connections.
    rcbehs = self.each_behavior.map {|beh| beh.to_rcsim(subowner)} # +
        # self.each_connection.map {|cxt| cxt.to_rcsim(subowner) }
    self.each_connection do |cnx|
        if !cnx.right.is_a?(RefObject) then
            rcbehs << cnx.to_rcsim(subowner)
        else
            # puts "cnx.left.object=#{cnx.left.object.fullname} cnx.right.object=#{cnx.right.object.fullname}"
            rcbehs << cnx.to_rcsim(subowner)
            if cnx.left.is_a?(RefObject) then
                sigL = cnx.left.object
                prtL = sigL.parent
                if prtL.is_a?(SystemT) and prtL.each_inout.any?{|e| e.object_id == sigL.object_id} then
                    # puts "write to right with sigL=#{sigL.fullname}."
                    rcbehs << Connection.new(cnx.right.clone,cnx.left.clone).to_rcsim(subowner)
                end
            end
        end
    end
    if rcbehs.any? then
        RCSim.rcsim_add_scope_behaviors(@rcscope,rcbehs)
    end

    # Create and add the programs.
    rcprogs = self.each_program.map {|prog| prog.to_rcsim(subowner)} 
    if rcprogs.any? then
        RCSim.rcsim_add_scope_codes(@rcscope,rcprogs);
    end

    return @rcscope
end

#to_refObject

Converts to a new reference.



1146
1147
1148
# File 'lib/HDLRuby/hruby_high.rb', line 1146

def to_ref
    return RefObject.new(this,self)
end

#to_userObject

Converts to a namespace user.



960
961
962
963
# File 'lib/HDLRuby/hruby_high.rb', line 960

def to_user
    # Already a user.
    return self
end