Method: HDLRuby::High::SystemI#initialize
- Defined in:
- lib/HDLRuby/hruby_high.rb
#initialize(name, systemT) ⇒ SystemI
Creates a new system instance of system type +systemT+ named +name+.
2203 2204 2205 2206 2207 2208 2209 2210 |
# File 'lib/HDLRuby/hruby_high.rb', line 2203 def initialize(name, systemT) # Initialize the system instance structure. super(name,systemT) # Sets the hdl-like access to the system instance. obj = self # For using the right self within the proc High.space_reg(name) { obj } end |