Method: Handshaker#input
- Defined in:
- lib/HDLRuby/hdr_samples/with_class.rb
#input ⇒ Object
Declares the signals used for input from the handshaker and do the connections of the upper SystemI
50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 |
# File 'lib/HDLRuby/hdr_samples/with_class.rb', line 50 def input ibuffer = iread_valid = iread_ready = iwrite_valid = iwrite_ready =nil type = @type buffer = @buffer read_valid = @read_valid read_ready = @read_ready write_valid = @write_valid write_ready = @write_ready HDLRuby::High.cur_system.open do # Declares the input signals ibuffer = type.input(HDLRuby.uniq_name) iread_valid = input(HDLRuby.uniq_name) iread_ready = input(HDLRuby.uniq_name) iwrite_valid = output(HDLRuby.uniq_name) iwrite_ready = output(HDLRuby.uniq_name) end @ibuffer = ibuffer @iread_valid = iread_valid @iread_ready = iread_ready @iwrite_valid = iwrite_valid @iwrite_ready = iwrite_ready end |