Class: String

Inherits:
Object
  • Object
show all
Defined in:
lib/HDLRuby/hruby_high.rb

Overview

Extends the String class for computing conversion to expression.

Instance Method Summary collapse

Instance Method Details

#to_exprObject

Convert to a new high-level string expression



5336
5337
5338
# File 'lib/HDLRuby/hruby_high.rb', line 5336

def to_expr
    return StringE.new(self)
end

#to_valueObject

Converts to a new high-level value.



5327
5328
5329
5330
5331
5332
5333
# File 'lib/HDLRuby/hruby_high.rb', line 5327

def to_value
    # Convert the string to a bit string.
    bstr = BitString.new(self)
    # Use it to create the new value.
    # return Value.new(Bit[bstr.width],bstr)
    return Value.new(Bit[self.length],bstr)
end

#to_value?Boolean

Tell if the expression can be converted to a value.

Returns:

  • (Boolean)


5322
5323
5324
# File 'lib/HDLRuby/hruby_high.rb', line 5322

def to_value?
    return true
end